import { createCanvas } from 'canvas';
import { initializeCanvas } from './index';
initializeCanvas(createCanvas);
export function initialize() {
    initializeCanvas(createCanvas);
}
//# sourceMappingURL=initializeCanvas.js.map